@@ -4,6 +4,7 @@ module Data.Tuple.HT (
    mapFst,
    mapSnd,
    swap,
+   sortPair,
    forcePair,
 
    -- * Triple
@@ -36,3 +37,7 @@ thd3 (_,_,x) = x
 {-# INLINE curry3 #-}
 curry3 :: ((a, b, c) -> d) -> a -> b -> c -> d
 curry3 f a b c = f (a,b,c)
+
+sortPair, _sortPairMinMax :: (Ord a) => (a,a) -> (a,a)
+sortPair (x,y) = if x<=y then (x,y) else (y,x)
+_sortPairMinMax (x,y) = (min x y, max x y)
